'use strict';

const _            = require('lodash');
const Promise      = require('bluebird');
const YAML         = require('js-yaml');
const Base = require('./../../kube-native/resource.base.controller');
const Release = require('./release');
const { parseFamiliarName } = require('@codefresh-io/docker-reference');

const TILLER_LABEL='OWNER=TILLER';

class ReleaseController extends Base {

    _ensureTillerExistOnNamespace(tillerServiceName = 'tiller-deploy'){
        return this._kubeManager.getServiceController(this._ns).describe(tillerServiceName)
            .catch((err) => {
                throw new Error(`Tiller not found in namespace ${this._ns}`);
            });
    }

    _getRelevantConfigmaps(name){
        const basicLabel = TILLER_LABEL;
        return this._kubeManager.getConfigmapController(this._ns)
            .get({labelSelector: name ? `${name},${basicLabel}` : basicLabel});
    }

    _readReleaseDataFromConfigmap(configmap){
        const rawName = configmap.getMetadata().name;
        const labels = configmap.getLabels();
        const dataAsBase64 = configmap.getProperty('data.release'); 
        return new Release(Object.assign({rawName, dataAsBase64}, labels));    
    }

    get() {
        const releases = [];
        return this._ensureTillerExistOnNamespace()
        .then(() => this._getRelevantConfigmaps())
            .map((cm) => {
                const release = this._readReleaseDataFromConfigmap(cm);
                return release.initRevisionData()
                .then(() => {
                    const name = release.name;
                    const releaseCandide = releases.find(re => re.name === name);

                    if (!releaseCandide) {
                        releases.push(release)
                    } else {
                        releaseCandide.updateRelease(release)
                    }
                });
                
            })
            .then(() => Promise.map(releases, (release => release.finalize())))
            .return(releases);
    }

    describe(name) {
        let finalRelease;
        return this._ensureTillerExistOnNamespace()
        .then(() => this._getRelevantConfigmaps(`NAME=${name}`))
        .map((cm) => {
            const release = this._readReleaseDataFromConfigmap(cm);
            const name = release.name;
            if(!finalRelease){
                finalRelease = release;
            } else {
                finalRelease.updateRelease(release);
            }
        })
        .then(() => finalRelease ? finalRelease.finalize() : undefined)
        .then(() => finalRelease);
    }

    create(resourceDescriptor) {
        throw new Error('Not supported yet');
    }

    delete(name) {
        throw new Error('Not supported yet');
    }

    update(name, resourceDescriptor) {
        throw new Error('Not supported yet');
    }

    publishEvent(name, data) {
        this.emit(name, data);
    }

    getTillerVersion(tillerServiceName = 'tiller-deploy') {
        return this._ensureTillerExistOnNamespace(tillerServiceName)
            .then(() => this._kubeManager.getServiceController(this._ns).describe(tillerServiceName))
            .then(service => _.chain(service.getServiceSelector()).map((v, k) => `${k}=${v}`).join(',').value())
            .then(labelSelector => this._kubeManager.getPodController(this._ns).get({
                labelSelector
            }))
            .then((pods) => {
                if(!pods){
                    throw new Error(`Could not find tiller pod`);
                }
                return pods[0].getImages()
            })
            .then(images => parseFamiliarName(images[0].name))
            .then(refernce => refernce.tag);
    }

}

module.exports = ReleaseController;